searchNodes=[{"type":"module","title":"Dune","doc":"A sandbox for Elixir to safely evaluate untrusted code from user input. Features only authorized modules and functions can be executed (see Dune.Allowlist.Default ) no access to environment variables, file system, network... code executed in an isolated process execution within configurable limits: timeout, maximum reductions and memory (inspired by Luerl ) captured standard output atoms, without atom leaks: parsing and runtime do not leak atoms (i.e. does not keep filling the atom table until the VM crashes) modules, without actual module creation: Dune does not let users define any actual module (would leak memory and modify the state of the VM globally), but defmodule simulates the basic behavior of a module, including private and recursive functions The list of modules and functions authorized by default is defined by the Dune.Allowlist.Default module, but this list can be extended and customized (at your own risk!) using Dune.Allowlist . If you need to keep the state between evaluations, you might consider Dune.Session .","ref":"Dune.html"},{"type":"function","title":"Dune.eval_quoted/2","doc":"Evaluates the quoted ast in the sandbox. Available options are detailed in Dune.Opts (parsing restrictions have no effect).. Returns a Dune.Success struct if the execution went successfully, a Dune.Failure else. Examples iex&gt; Dune . eval_quoted ( quote do : [ 1 , 2 ] ++ [ 3 , 4 ] ) % Dune.Success { inspected : &quot;[1, 2, 3, 4]&quot; , stdio : &quot;&quot; , value : [ 1 , 2 , 3 , 4 ] } iex&gt; Dune . eval_quoted ( quote do : System . get_env ( ) ) % Dune.Failure { message : &quot;** (DuneRestrictedError) function System.get_env/0 is restricted&quot; , type : :restricted } iex&gt; Dune . eval_quoted ( quote do : Process . sleep ( 500 ) ) % Dune.Failure { message : &quot;Execution timeout - 50ms&quot; , type : :timeout }","ref":"Dune.html#eval_quoted/2"},{"type":"function","title":"Dune.eval_string/2","doc":"Evaluates the string in the sandbox. Available options are detailed in Dune.Opts . Returns a Dune.Success struct if the execution went successfully, a Dune.Failure else. Examples iex&gt; Dune . eval_string ( &quot;IO.puts(&#39;Hello world!&#39;)&quot; ) % Dune.Success { inspected : &quot;:ok&quot; , stdio : &quot;Hello world! \\n &quot; , value : :ok } iex&gt; Dune . eval_string ( &quot;File.cwd!()&quot; ) % Dune.Failure { message : &quot;** (DuneRestrictedError) function File.cwd!/0 is restricted&quot; , type : :restricted } iex&gt; Dune . eval_string ( &quot;List.duplicate(:spam, 100_000)&quot; ) % Dune.Failure { message : &quot;Execution stopped - memory limit exceeded&quot; , stdio : &quot;&quot; , type : :memory } iex&gt; Dune . eval_string ( &quot;Foo.bar()&quot; ) % Dune.Failure { message : &quot;** (UndefinedFunctionError) function Foo.bar/0 is undefined (module Foo is not available)&quot; , type : :exception } iex&gt; Dune . eval_string ( &quot;][&quot; ) % Dune.Failure { message : &quot;unexpected token: ]&quot; , type : :parsing } Atoms used during parsing and execution might be transformed to prevent atom leaks: iex&gt; Dune . eval_string ( &quot;some_variable = IO.inspect(:some_atom)&quot; ) % Dune.Success { inspected : &quot;:some_atom&quot; , stdio : &quot;:some_atom \\n &quot; , value : :a__Dune_atom_2__ } The value field shows the actual runtime value, but inspected and stdio are safe to display to the user.","ref":"Dune.html#eval_string/2"},{"type":"function","title":"Dune.string_to_quoted/2","doc":"Returns the AST corresponding to the provided string , without leaking atoms. Available options are detailed in Dune.Opts (runtime restrictions have no effect). Returns a Dune.Success struct if the execution went successfully, a Dune.Failure else. Examples iex&gt; Dune . string_to_quoted ( &quot;1 + 2&quot; ) % Dune.Success { inspected : &quot;{:+, [line: 1], [1, 2]}&quot; , stdio : &quot;&quot; , value : { :+ , [ line : 1 ] , [ 1 , 2 ] } } iex&gt; Dune . string_to_quoted ( &quot;[invalid&quot; ) % Dune.Failure { stdio : &quot;&quot; , message : &quot;missing terminator: ] (for \\&quot; [ \\&quot; starting at line 1)&quot; , type : :parsing } The pretty option can make the AST more readable by adding newlines to inspected : iex&gt; Dune . string_to_quoted ( &quot;IO.puts(:hello)&quot; , pretty : true ) . inspected &quot;{{:., [line: 1], [{:__aliases__, [line: 1], [:IO]}, :puts]}, [line: 1], \\n [:hello]}&quot; iex&gt; Dune . string_to_quoted ( &quot;IO.puts(:hello)&quot; ) . inspected &quot;{{:., [line: 1], [{:__aliases__, [line: 1], [:IO]}, :puts]}, [line: 1], [:hello]}&quot; Since the code isn't executed, there is no allowlist restriction: iex&gt; Dune . string_to_quoted ( &quot;System.halt()&quot; ) % Dune.Success { inspected : &quot;{{:., [line: 1], [{:__aliases__, [line: 1], [:System]}, :halt]}, [line: 1], []}&quot; , stdio : &quot;&quot; , value : { { :. , [ line : 1 ] , [ { :__aliases__ , [ line : 1 ] , [ :System ] } , :halt ] } , [ line : 1 ] , [ ] } } Atoms might be transformed during parsing to prevent atom leaks: iex&gt; Dune . string_to_quoted ( &quot;some_variable = :some_atom&quot; ) % Dune.Success { inspected : &quot;{:=, [line: 1], [{:some_variable, [line: 1], nil}, :some_atom]}&quot; , stdio : &quot;&quot; , value : { := , [ line : 1 ] , [ { :a__Dune_atom_1__ , [ line : 1 ] , nil } , :a__Dune_atom_2__ ] } } The value field shows the actual runtime value, but inspected is safe to display to the user.","ref":"Dune.html#string_to_quoted/2"},{"type":"behaviour","title":"Dune.Allowlist","doc":"Behaviour to customize the modules and functions that are allowed or restricted. Warning: security considerations The default implementation is Dune.Allowlist.Default , and should only allow safe functions: no atom leaks, no execution of arbitrary code, no access to the filesystem / network... Defining or extending a custom Dune.Allowlist module can introduce security risks or bugs. Please also note that using custom allowlists is still experimental and the API for it might change faster than the rest of the library. Defining a new allowlist In order to define a custom allowlist from scratch, use Dune.Allowlist can be used: defmodule CustomAllowlist do use Dune.Allowlist allow Kernel , only : [ :+ , :* , :- , :/ , :div , :rem ] end Dune . eval_string ( &quot;4 + 9&quot; , allowlist : CustomAllowlist ) Extending an existing allowlist Defining an allowlist from scratch can be both daunting and risky. It is possible to extend an exisiting allowlist instead using the extend option: defmodule ExtendedAllowlist do use Dune.Allowlist , extend : Dune.Allowlist.Default allow SomeModule , only : [ :authorized ] end Dune . eval_string ( &quot;SomeModule.authorized(123)&quot; , allowlist : ExtendedAllowlist ) Note: currently, it is not possible to add or restrict functions from modules that have already been specified. Documentation generation The list of modules and functions with their status can be generated in the @moduledoc . An example can be found in the Dune.Allowlist.Default documentation. If the __DUNE_ALLOWLIST_FUNCTIONS__ string is found in the @moduledoc string, it will be replaced. defmodule CustomAllowlist do @moduledoc &quot;&quot;&quot; Only allows simple arithmetic ## Allowlist functions __DUNE_ALLOWLIST_FUNCTIONS__ &quot;&quot;&quot; use Dune.Allowlist allow Kernel , only : [ :+ , :* , :- , :/ , :div , :rem ] end","ref":"Dune.Allowlist.html"},{"type":"macro","title":"Dune.Allowlist.allow/2","doc":"Adds a new module to the allowlist and specifices which functions to use. The module must not be already specified in the allowlist. Must be called after use Dune.Allowlist . Examples # allow all functions in a module allow Time , :all # only allow specific functions allow Function , only : [ :identity ] # exclude specific functions allow Calendar , except : [ :put_time_zone_database ] Note: only and except will cover all arities if several functions share a name.","ref":"Dune.Allowlist.html#allow/2"},{"type":"function","title":"Dune.Allowlist.ensure_implements_behaviour!/1","doc":"Validates the fact that a module implements the Dune.Allowlist behaviour. Raises if not the case. Examples iex&gt; Dune.Allowlist . ensure_implements_behaviour! ( DoesNotExists ) ** (ArgumentError) could not load module DoesNotExists due to reason :nofile iex&gt; Dune.Allowlist . ensure_implements_behaviour! ( List ) ** (ArgumentError) List does not implement the Dune.Allowlist behaviour","ref":"Dune.Allowlist.html#ensure_implements_behaviour!/1"},{"type":"callback","title":"Dune.Allowlist.fun_status/3","doc":"Returns the trust status of a function or macro, specified as a module , fun_name and arity ( mfa ): :allowed if can be safely use :restricted if its usage should be forbidden a {:shimmed, module, function_name} if the function call should be replaced with an alternative implementation","ref":"Dune.Allowlist.html#c:fun_status/3"},{"type":"type","title":"Dune.Allowlist.status/0","doc":"","ref":"Dune.Allowlist.html#t:status/0"},{"type":"module","title":"Dune.Allowlist.Default","doc":"The default Dune.Allowlist module to be used to allow or restrict functions and macros that can be safely executed. Examples iex&gt; Dune.Allowlist.Default . fun_status ( Kernel , :+ , 2 ) :allowed iex&gt; Dune.Allowlist.Default . fun_status ( String , :to_atom , 1 ) :restricted iex&gt; Dune.Allowlist.Default . fun_status ( Atom , :to_string , 1 ) { :shimmed , Dune.Shims.Atom , :to_string } iex&gt; Dune.Allowlist.Default . fun_status ( Kernel , :foo , 1 ) :undefined_function iex&gt; Dune.Allowlist.Default . fun_status ( Bar , :foo , 1 ) :undefined_module iex&gt; Dune.Allowlist.Default . fun_status ( Kernel.SpecialForms , :quote , 2 ) :restricted Allowed modules / functions Access Allowed : all , at , at! , elem , fetch , fetch! , filter , get , get_and_update , key , key! , pop , slice Atom Alernative implementation : to_charlist , to_string Restricted : to_char_list Base Allowed : decode16 , decode16! , decode32 , decode32! , decode64 , decode64! , encode16 , encode32 , encode64 , hex_decode32 , hex_decode32! , hex_encode32 , url_decode64 , url_decode64! , url_encode64 Bitwise Allowed : &amp;&amp;&amp; , &lt;&lt;&lt; , &gt;&gt;&gt; , ^^^ , __using__ , band , bnot , bor , bsl , bsr , bxor , ||| , ~~~ Calendar Allowed : compatible_calendars? , get_time_zone_database , strftime , truncate Restricted : put_time_zone_database Calendar.ISO Allowed : add_day_fraction_to_iso_days , date_from_iso_days , date_to_iso_days , date_to_string , datetime_to_string , day_of_era , day_of_week , day_of_year , day_rollover_relative_to_midnight_utc , days_in_month , from_unix , gregorian_seconds_to_iso_days , iso_days_to_beginning_of_day , iso_days_to_day_of_week , iso_days_to_end_of_day , iso_days_to_unit , leap_year? , months_in_year , naive_datetime_from_iso_days , naive_datetime_to_iso_days , naive_datetime_to_string , offset_to_string , parse_date , parse_naive_datetime , parse_time , parse_utc_datetime , quarter_of_year , time_from_day_fraction , time_to_day_fraction , time_to_string , time_unit_to_precision , valid_date? , valid_time? , year_of_era Date Allowed : __struct__ , add , after? , before? , beginning_of_month , beginning_of_week , compare , convert , convert! , day_of_era , day_of_week , day_of_year , days_in_month , diff , end_of_month , end_of_week , from_erl , from_erl! , from_gregorian_days , from_iso8601 , from_iso8601! , leap_year? , months_in_year , new , new! , quarter_of_year , range , to_erl , to_gregorian_days , to_iso8601 , to_iso_days , to_string , utc_today , year_of_era DateTime Allowed : __struct__ , add , after? , before? , compare , convert , convert! , diff , from_gregorian_seconds , from_iso8601 , from_naive , from_naive! , from_unix , from_unix! , new , new! , now , now! , shift_zone , shift_zone! , to_date , to_gregorian_seconds , to_iso8601 , to_naive , to_string , to_time , to_unix , truncate , utc_now Enum Allowed : all? , any? , at , chunk , chunk_by , chunk_every , chunk_while , concat , count , count_until , dedup , dedup_by , drop , drop_every , drop_while , each , empty? , fetch , fetch! , filter , filter_map , find , find_index , find_value , flat_map , flat_map_reduce , frequencies , frequencies_by , group_by , intersperse , into , map , map_every , map_intersperse , map_reduce , max , max_by , member? , min , min_by , min_max , min_max_by , partition , product , random , reduce , reduce_while , reject , reverse , reverse_slice , scan , shuffle , slice , slide , sort , sort_by , split , split_while , split_with , sum , take , take_every , take_random , take_while , to_list , uniq , uniq_by , unzip , with_index , zip , zip_reduce , zip_with Alernative implementation : join , map_join Float Allowed : ceil , floor , max_finite , min_finite , parse , pow , ratio , round , to_char_list , to_charlist , to_string Function Allowed : identity Restricted : capture , info IO Allowed : chardata_to_string , iodata_length , iodata_to_binary Alernative implementation : inspect , puts Restricted : binread , binstream , binwrite , each_binstream , each_stream , getn , gets , read , stream , warn , warn_once , write Integer Allowed : digits , extended_gcd , floor_div , gcd , is_even , is_odd , mod , parse , pow , to_char_list , to_charlist , to_string , undigits Kernel Allowed : ! , != , !== , &amp;&amp; , * , + , ++ , - , -- , .. , ..// , / , &lt; , &lt;= , &lt;&gt; , == , === , =~ , &gt; , &gt;= , abs , and , binary_part , bit_size , byte_size , ceil , div , elem , floor , get_and_update_in , get_in , hd , if , in , is_atom , is_binary , is_bitstring , is_boolean , is_exception , is_float , is_function , is_integer , is_list , is_map , is_map_key , is_nil , is_number , is_reference , is_struct , is_tuple , length , make_ref , map_size , match? , max , min , not , or , pop_in , put_elem , put_in , raise , rem , round , self , sigil_C , sigil_D , sigil_N , sigil_R , sigil_S , sigil_T , sigil_U , sigil_c , sigil_r , sigil_s , tap , then , tl , trunc , tuple_size , unless , update_in , |&gt; , || Alernative implementation : apply , dbg , inspect , sigil_W , sigil_w , throw , to_charlist , to_string Restricted : ** , @ , alias! , binary_slice , binding , def , defdelegate , defexception , defguard , defguardp , defimpl , defmacro , defmacrop , defmodule , defoverridable , defp , defprotocol , defstruct , destructure , exit , function_exported? , is_pid , is_port , macro_exported? , node , reraise , send , spawn , spawn_link , spawn_monitor , struct , struct! , to_char_list , use , var! Kernel.SpecialForms Allowed : %{} , :: , &lt;&lt;&gt;&gt; , = , ^ , __aliases__ , case , cond , fn , for , with , {} Restricted : % , &amp; , . , __CALLER__ , __DIR__ , __ENV__ , __MODULE__ , __STACKTRACE__ , __block__ , alias , import , quote , receive , require , super , try , unquote , unquote_splicing Keyword Allowed : delete , delete_first , drop , equal? , fetch , fetch! , filter , from_keys , get , get_and_update , get_and_update! , get_lazy , get_values , has_key? , keys , keyword? , map , merge , new , pop , pop! , pop_first , pop_lazy , pop_values , put , put_new , put_new_lazy , reject , replace , replace! , replace_lazy , size , split , split_with , take , to_list , update , update! , validate , validate! , values List Allowed : ascii_printable? , delete , delete_at , duplicate , first , flatten , foldl , foldr , improper? , insert_at , keydelete , keyfind , keyfind! , keymember? , keyreplace , keysort , keystore , keytake , last , myers_difference , pop_at , replace_at , starts_with? , to_charlist , to_float , to_integer , to_string , to_tuple , update_at , wrap , zip Restricted : to_atom , to_existing_atom Map Allowed : delete , drop , equal? , fetch , fetch! , filter , from_keys , from_struct , get , get_and_update , get_and_update! , get_lazy , has_key? , intersect , keys , map , merge , new , pop , pop! , pop_lazy , put , put_new , put_new_lazy , reject , replace , replace! , replace_lazy , size , split , split_with , take , to_list , update , update! , values MapSet Allowed : __struct__ , delete , difference , disjoint? , equal? , filter , intersection , member? , new , put , reject , size , split_with , subset? , symmetric_difference , to_list , union NaiveDateTime Allowed : __struct__ , add , after? , before? , beginning_of_day , compare , convert , convert! , diff , end_of_day , from_erl , from_erl! , from_gregorian_seconds , from_iso8601 , from_iso8601! , local_now , new , new! , to_date , to_erl , to_gregorian_seconds , to_iso8601 , to_string , to_time , truncate , utc_now Process Allowed : sleep Restricted : alias , alive? , cancel_timer , delete , demonitor , exit , flag , get , get_keys , group_leader , hibernate , info , link , list , monitor , put , read_timer , register , registered , send , send_after , spawn , unalias , unlink , unregister , whereis Range Allowed : __struct__ , disjoint? , new , range? , shift , size , split , to_list Regex Allowed : __struct__ , compile , compile! , escape , match? , named_captures , names , opts , re_pattern , recompile , recompile! , regex? , replace , run , scan , source , split , unescape_map , version Stream Allowed : __struct__ , chunk , chunk_by , chunk_every , chunk_while , concat , cycle , dedup , dedup_by , drop , drop_every , drop_while , duplicate , each , filter , filter_map , flat_map , intersperse , interval , into , iterate , map , map_every , reject , repeatedly , resource , run , scan , take , take_every , take_while , timer , transform , unfold , uniq , uniq_by , with_index , zip , zip_with String Allowed : at , bag_distance , capitalize , chunk , codepoints , contains? , downcase , duplicate , ends_with? , equivalent? , first , graphemes , jaro_distance , last , length , ljust , lstrip , match? , myers_difference , next_codepoint , next_grapheme , next_grapheme_size , normalize , pad_leading , pad_trailing , printable? , replace , replace_leading , replace_prefix , replace_suffix , replace_trailing , reverse , rjust , rstrip , slice , split , split_at , splitter , starts_with? , strip , to_char_list , to_charlist , to_float , to_integer , trim , trim_leading , trim_trailing , upcase , valid? , valid_character? Restricted : to_atom , to_existing_atom Time Allowed : __struct__ , add , after? , before? , compare , convert , convert! , diff , from_erl , from_erl! , from_iso8601 , from_iso8601! , from_seconds_after_midnight , new , new! , to_erl , to_iso8601 , to_seconds_after_midnight , to_string , truncate , utc_now Tuple Allowed : append , delete_at , duplicate , insert_at , product , sum , to_list URI Allowed : __struct__ , append_path , append_query , char_reserved? , char_unescaped? , char_unreserved? , decode , decode_query , decode_www_form , default_port , encode , encode_query , encode_www_form , merge , new , new! , parse , query_decoder , to_string :array Allowed : default , fix , foldl , foldr , from_list , from_orddict , get , is_array , is_fix , map , new , relax , reset , resize , set , size , sparse_foldl , sparse_foldr , sparse_map , sparse_size , sparse_to_list , sparse_to_orddict , to_list , to_orddict :binary Allowed : at , bin_to_list , compile_pattern , copy , decode_hex , decode_unsigned , encode_hex , encode_unsigned , first , last , list_to_bin , longest_common_prefix , longest_common_suffix , match , matches , part , referenced_byte_size , replace , split :erlang Allowed : * , + , ++ , - , -- , / , /= , &lt; , =/= , =:= , =&lt; , == , &gt; , &gt;= , abs , adler32 , adler32_combine , and , append_element , band , binary_part , binary_to_float , binary_to_integer , binary_to_list , bit_size , bitstring_to_list , bnot , bor , bsl , bsr , bxor , byte_size , ceil , convert_time_unit , crc32 , crc32_combine , date , delete_element , div , element , float , float_to_binary , float_to_list , floor , hd , insert_element , integer_to_binary , integer_to_list , iolist_size , iolist_to_binary , iolist_to_iovec , is_atom , is_binary , is_bitstring , is_boolean , is_float , is_function , is_integer , is_list , is_map , is_map_key , is_number , is_pid , is_port , is_record , is_reference , is_tuple , length , list_to_binary , list_to_bitstring , list_to_float , list_to_integer , localtime , localtime_to_universaltime , make_ref , make_tuple , map_get , map_size , max , md5 , md5_final , md5_init , md5_update , min , monotonic_time , not , or , phash2 , ref_to_list , rem , round , setelement , size , split_binary , system_time , time , time_offset , timestamp , tl , trunc , tuple_size , tuple_to_list , unique_integer , universaltime , universaltime_to_localtime , xor Alernative implementation : apply Restricted : ! , alias , alloc_info , alloc_sizes , append , atom_to_binary , atom_to_list , binary_to_atom , binary_to_existing_atom , binary_to_term , bump_reductions , call_on_load_function , cancel_timer , check_old_code , check_process_code , decode_packet , delay_trap , delete_module , demonitor , disconnect_node , display , display_string , dist_ctrl_get_data , dist_ctrl_get_data_notification , dist_ctrl_get_opt , dist_ctrl_input_handler , dist_ctrl_put_data , dist_ctrl_set_opt , dist_get_stat , dmonitor_node , dt_append_vm_tag_data , dt_get_tag , dt_get_tag_data , dt_prepend_vm_tag_data , dt_put_tag , dt_restore_tag , dt_spread_tag , erase , error , exit , exit_signal , external_size , finish_after_on_load , finish_loading , format_cpu_topology , fun_info , fun_info_mfa , fun_to_list , function_exported , garbage_collect , garbage_collect_message_area , gather_gc_info_result , get , get_cookie , get_keys , get_module_info , group_leader , halt , has_prepared_code_on_load , hibernate , is_alive , is_builtin , is_process_alive , link , list_to_atom , list_to_existing_atom , list_to_pid , list_to_port , list_to_ref , list_to_tuple , load_module , load_nif , loaded , make_fun , match_spec_test , memory , module_loaded , monitor , monitor_node , nif_error , node , nodes , now , open_port , phash , pid_to_list , port_call , port_close , port_command , port_connect , port_control , port_get_data , port_info , port_set_data , port_to_list , ports , posixtime_to_universaltime , pre_loaded , prepare_loading , process_display , process_flag , process_info , processes , purge_module , put , raise , read_timer , register , registered , resume_process , self , send , send_after , send_nosuspend , seq_trace , seq_trace_info , seq_trace_print , set_cookie , set_cpu_topology , setnode , spawn , spawn_link , spawn_monitor , spawn_opt , spawn_request , spawn_request_abandon , start_timer , statistics , subtract , suspend_process , system_flag , system_info , system_monitor , system_profile , term_to_binary , term_to_iovec , throw , trace , trace_delivered , trace_info , trace_pattern , unalias , universaltime_to_posixtime , unlink , unregister , whereis , yield :erts_debug Allowed : flat_size , same , size , size_shared Restricted : alloc_blocks_size , breakpoint , copy_shared , df , dirty , dirty_cpu , dirty_io , dis_to_file , disassemble , display , dist_ext_to_term , get_internal_state , ic , instructions , interpreter_size , lc_graph , lc_graph_merge , lc_graph_to_dot , lcnt_clear , lcnt_collect , lcnt_control , map_info , set_internal_state :gb_sets Allowed : add , add_element , balance , del_element , delete , delete_any , difference , empty , filter , fold , from_list , from_ordset , insert , intersection , is_disjoint , is_element , is_empty , is_member , is_set , is_subset , iterator , iterator_from , largest , new , next , singleton , size , smallest , subtract , take_largest , take_smallest , to_list , union :gb_trees Allowed : balance , delete , delete_any , empty , enter , from_orddict , get , insert , is_defined , is_empty , iterator , iterator_from , keys , largest , lookup , map , next , size , smallest , take , take_any , take_largest , take_smallest , to_list , update , values :lists Allowed : all , any , append , concat , delete , droplast , dropwhile , duplicate , enumerate , filter , filtermap , flatlength , flatmap , flatten , foldl , foldr , foreach , join , keydelete , keyfind , keymap , keymember , keymerge , keyreplace , keysearch , keysort , keystore , keytake , last , map , mapfoldl , mapfoldr , max , member , merge , merge3 , min , nth , nthtail , partition , prefix , reverse , rkeymerge , rmerge , rmerge3 , rukeymerge , rumerge , rumerge3 , search , seq , sort , split , splitwith , sublist , subtract , suffix , sum , takewhile , ukeymerge , ukeysort , umerge , umerge3 , uniq , unzip , unzip3 , usort , zf , zip , zip3 , zipwith , zipwith3 :maps Allowed : filter , filtermap , find , fold , foreach , from_keys , from_list , get , groups_from_list , intersect , intersect_with , is_iterator_valid , is_key , iterator , keys , map , merge , merge_with , new , next , put , remove , size , take , to_list , update , update_with , values , with , without :math Allowed : acos , acosh , asin , asinh , atan , atan2 , atanh , ceil , cos , cosh , erf , erfc , exp , floor , fmod , log , log10 , log2 , pi , pow , sin , sinh , sqrt , tan , tanh , tau :orddict Allowed : append , append_list , erase , fetch , fetch_keys , filter , find , fold , from_list , is_empty , is_key , map , merge , new , size , store , take , to_list , update , update_counter :ordsets Allowed : add_element , del_element , filter , fold , from_list , intersection , is_disjoint , is_element , is_empty , is_set , is_subset , new , size , subtract , to_list , union :proplists Allowed : append_values , compact , delete , expand , from_map , get_all_values , get_bool , get_keys , get_value , is_defined , lookup , lookup_all , normalize , property , split , substitute_aliases , substitute_negations , to_map , unfold :queue Allowed : all , any , cons , daeh , delete , delete_r , delete_with , delete_with_r , drop , drop_r , filter , filtermap , fold , from_list , get , get_r , head , in , in_r , init , is_empty , is_queue , join , lait , last , len , liat , member , new , out , out_r , peek , peek_r , reverse , snoc , split , tail , to_list :rand Allowed : bc64 , bytes , bytes_s , export_seed , export_seed_s , exro928_jump_2pow20 , exro928_jump_2pow512 , exro928_next , exro928_next_state , exro928_seed , exsp_jump , exsp_next , float2str , format_jumpconst58 , jump , make_float , mwc59 , mwc59_float , mwc59_seed , mwc59_value , mwc59_value32 , normal , normal_s , seed , seed58 , seed_s , splitmix64_next , uniform , uniform_real , uniform_real_s , uniform_s :string Allowed : casefold , centre , chars , chomp , chr , concat , copies , cspan , equal , find , is_empty , join , left , len , length , lexemes , list_to_float , list_to_integer , lowercase , next_codepoint , next_grapheme , nth_lexeme , pad , prefix , rchr , replace , reverse , right , rstr , slice , span , split , str , strip , sub_string , sub_word , substr , take , titlecase , to_float , to_graphemes , to_integer , to_lower , to_upper , tokens , trim , uppercase , words :unicode Allowed : bin_is_7bit , bom_to_encoding , characters_to_binary , characters_to_binary_int , characters_to_list , characters_to_list_int , characters_to_nfc_binary , characters_to_nfc_list , characters_to_nfd_binary , characters_to_nfd_list , characters_to_nfkc_binary , characters_to_nfkc_list , characters_to_nfkd_binary , characters_to_nfkd_list , encoding_to_bom :zlib Allowed : compress , gunzip , gzip , uncompress , unzip , zip Restricted : adler32 , adler32_combine , close , crc32 , crc32_combine , deflate , deflateEnd , deflateInit , deflateParams , deflateReset , deflateSetDictionary , getBufSize , inflate , inflateChunk , inflateEnd , inflateGetDictionary , inflateInit , inflateReset , inflateSetDictionary , on_load , open , safeInflate , setBufSize , set_controlling_process","ref":"Dune.Allowlist.Default.html"},{"type":"function","title":"Dune.Allowlist.Default.fun_status/3","doc":"Implements Dune.Allowlist.fun_status/3","ref":"Dune.Allowlist.Default.html#fun_status/3"},{"type":"module","title":"Dune.Failure","doc":"A struct returned when Dune parsing or evaluation fails. Fields: message (string): the error message to display to the user type (atom): the nature of the error stdio (string): captured standard output","ref":"Dune.Failure.html"},{"type":"type","title":"Dune.Failure.error_type/0","doc":"","ref":"Dune.Failure.html#t:error_type/0"},{"type":"type","title":"Dune.Failure.t/0","doc":"","ref":"Dune.Failure.html#t:t/0"},{"type":"module","title":"Dune.Opts","doc":"Defines and validates the options for Dune . The available options are explained below: Parsing restriction options atom_pool_size : Defines the maximum total number of atoms that can be created. Must be an integer &gt;= 0 . Defaults to 5000 . See the section below for more information. max_length : Defines the maximum length of code strings that can be parsed. Defaults to 5000 . Execution restriction options allowlist : Defines which module and functions are considered safe or restricted. Should be a module implementing the Dune.Allowlist behaviour. Defaults to Dune.Allowlist.Default . max_heap_size : Limits the memory usage of the evaluation process using the max_heap_size flag . Should be an integer &gt; 0 . Defaults to 30_000 . max_reductions : Limits the number of CPU cycles of the evaluation process. The erlang pre-emptive scheduler is using reductions to measure work being done by processes, which is useful to prevent users to run CPU intensive code such as infinite loops. Should be an integer &gt; 0 . Defaults to 30_000 . timeout : Limits the time the evaluation process is authorized to run (in milliseconds). Should be an integer &gt; 0 . Defaults to 50 . The evaluation process will still need to parse and execute the sanitized AST, so using too low limits here would leave only a small margin to actually run user code. Other options pretty : Use pretty printing when inspecting the result. Should be a boolean. Defaults to false . Extra note about atom_pool_size Atoms are reused from one evaluation to the other so the total is not expected to grow. Atoms will not be leaked. Also, the atom pool is actually split into several pools: regular atoms, module names, unused variable names, ... So defining a value of 100 does not mean that 100 atoms will be available, but rather 25 of each type. Atoms being very lightweight, there is no need to use a low value, as long as there is an upper bound preventing atom leaks.","ref":"Dune.Opts.html"},{"type":"function","title":"Dune.Opts.validate!/1","doc":"Validates untrusted options from a keyword or a map and returns a Dune.Opts struct. Examples iex&gt; Dune.Opts . validate! ( [ ] ) % Dune.Opts { allowlist : Dune.Allowlist.Default , atom_pool_size : 5000 , max_heap_size : 50000 , max_length : 5000 , max_reductions : 30000 , pretty : false , timeout : 50 } iex&gt; Dune.Opts . validate! ( atom_pool_size : 10 ) % Dune.Opts { atom_pool_size : 10 , allowlist : Dune.Allowlist.Default } iex&gt; Dune.Opts . validate! ( atom_pool_size : - 10 ) ** (ArgumentError) atom_pool_size should be an integer &gt;= 0 iex&gt; Dune.Opts . validate! ( max_length : 0 ) ** (ArgumentError) atom_pool_size should be an integer &gt; 0 iex&gt; Dune.Opts . validate! ( allowlist : DoesNotExists ) ** (ArgumentError) could not load module DoesNotExists due to reason :nofile iex&gt; Dune.Opts . validate! ( allowlist : List ) ** (ArgumentError) List does not implement the Dune.Allowlist behaviour iex&gt; Dune.Opts . validate! ( max_reductions : 10_000 , max_heap_size : 10_000 , timeout : 20 ) % Dune.Opts { max_heap_size : 10_000 , max_reductions : 10_000 , timeout : 20 } iex&gt; Dune.Opts . validate! ( max_heap_size : 0 ) ** (ArgumentError) max_heap_size should be an integer &gt; 0 iex&gt; Dune.Opts . validate! ( max_reductions : 0 ) ** (ArgumentError) max_reductions should be an integer &gt; 0 iex&gt; Dune.Opts . validate! ( timeout : &quot;55&quot; ) ** (ArgumentError) timeout should be an integer &gt; 0 iex&gt; Dune.Opts . validate! ( pretty : :maybe ) ** (ArgumentError) pretty should be a boolean","ref":"Dune.Opts.html#validate!/1"},{"type":"type","title":"Dune.Opts.t/0","doc":"","ref":"Dune.Opts.html#t:t/0"},{"type":"module","title":"Dune.Session","doc":"Sessions provide a way to evaluate code and keep state (bindings, modules...) between evaluations. Use Dune.eval_string/2 to execute code as a one-off Use Dune.Session.eval_string/3 to execute consecutive code blocks Dune.Session could be used to implement something like a safe IEx shell, or to compile a module once and call it several times without the overhead of parsing. Dune.Session is also a struct that is used to store the state of an evaluation. Only the following fields are public: last_result : contains the result of the last evaluation, or nil for empty sessions Other fields are private and shouldn't be accessed directly.","ref":"Dune.Session.html"},{"type":"function","title":"Dune.Session.eval_string/3","doc":"Evaluates the provided string in the context of the session and returns a new session. The result will be available in the last_result key. In case of a success, the variable bindings or created modules will be saved in the session. In case of a failure, the rest of the session state won't be updated, so it is possible to keep executing instructions after a failure Examples iex&gt; Dune.Session . new ( ) ...&gt; |&gt; Dune.Session . eval_string ( &quot;x = 1&quot; ) ...&gt; |&gt; Dune.Session . eval_string ( &quot;x + 2&quot; ) # Dune.Session &lt; last_result : % Dune.Success { value : 3 , inspected : &quot;3&quot; , stdio : &quot;&quot; } , ... &gt; iex&gt; Dune.Session . new ( ) ...&gt; |&gt; Dune.Session . eval_string ( &quot;x = 1&quot; ) ...&gt; |&gt; Dune.Session . eval_string ( &quot;x = x / 0&quot; ) # will fail, but the previous state is kept ...&gt; |&gt; Dune.Session . eval_string ( &quot;x + 2&quot; ) # Dune.Session &lt; last_result : % Dune.Success { value : 3 , inspected : &quot;3&quot; , stdio : &quot;&quot; } , ... &gt;","ref":"Dune.Session.html#eval_string/3"},{"type":"function","title":"Dune.Session.new/0","doc":"Returns a new empty session. Examples iex&gt; Dune.Session . new ( ) # Dune.Session &lt; last_result : nil , ... &gt;","ref":"Dune.Session.html#new/0"},{"type":"opaque","title":"Dune.Session.private_compile_env/0","doc":"","ref":"Dune.Session.html#t:private_compile_env/0"},{"type":"opaque","title":"Dune.Session.private_env/0","doc":"","ref":"Dune.Session.html#t:private_env/0"},{"type":"type","title":"Dune.Session.t/0","doc":"The type of a Dune.Session .","ref":"Dune.Session.html#t:t/0"},{"type":"module","title":"Dune.Success","doc":"A struct returned when Dune evaluation succeeds. Fields: value (term): the value which was actually returned at runtime. Should not be displayed to the user, might be different from what the user expects. inspected (string): safely inspected value to be displayed to the user stdio (string): captured standard output value contains the actual value used at runtime, so atoms will be different from the ones displayed to the user (see Dune.eval_string/2 ).","ref":"Dune.Success.html"},{"type":"type","title":"Dune.Success.t/0","doc":"","ref":"Dune.Success.html#t:t/0"},{"type":"extras","title":"Dune","doc":"A sandbox for Elixir to safely evaluate untrusted code from user input. Try it out on our online playground! Dune can be useful to develop playgrounds, online REPL, coding games, or customizable business logic. Warning: Dune cannot offer strong security guarantees (see the Security guarantees section below). Besides, it is still early stage: expect bugs and vulnerabilities.","ref":"readme.html"},{"type":"extras","title":"Dune - Features","doc":"allowlist mechanism (customizable) to restrict execution to safe modules and functions: no access to environment variables, file system, network... code executed in an isolated process execution within configurable limits: timeout, maximum reductions and memory (inspired by Luerl ) captured standard output atoms, without atom leaks: parsing and runtime do not leak atoms (i.e. does not keep filling the atom table until the VM crashes) modules, without actual module creation: Dune does not let users define any actual module (would leak memory and modify the state of the VM globally), but defmodule simulates the basic behavior of a module, including private and recursive functions iex&gt; Dune . eval_string ( &quot;IO.puts(&#39;Hello world!&#39;)&quot; ) % Dune.Success { inspected : &quot;:ok&quot; , stdio : &quot;Hello world! \\n &quot; , value : :ok } iex&gt; Dune . eval_string ( &quot;File.cwd!()&quot; ) % Dune.Failure { message : &quot;** (DuneRestrictedError) function File.cwd!/0 is restricted&quot; , type : :restricted } iex&gt; Dune . eval_string ( &quot;List.duplicate(:spam, 100_000)&quot; ) % Dune.Failure { message : &quot;Execution stopped - memory limit exceeded&quot; , stdio : &quot;&quot; , type : :memory } iex&gt; Dune . eval_string ( &quot;Enum.product(1..100_000)&quot; ) % Dune.Failure { message : &quot;Execution stopped - reductions limit exceeded&quot; , stdio : &quot;&quot; , type : :reductions } The list of modules and functions authorized by default is defined by the Dune.Allowlist.Default module, but this list can be extended and customized (at your own risk!) using Dune.Allowlist . If you need to keep the state between evaluations, you might consider Dune.Session : iex&gt; Dune.Session . new ( ) ...&gt; |&gt; Dune.Session . eval_string ( &quot;x = 1&quot; ) ...&gt; |&gt; Dune.Session . eval_string ( &quot;x + 2&quot; ) # Dune.Session &lt; last_result : % Dune.Success { inspected : &quot;3&quot; , stdio : &quot;&quot; , value : 3 } , ... &gt; Dune.string_to_quoted/2 returns the AST corresponding to the provided string , without leaking atoms: iex&gt; Dune . string_to_quoted ( &quot;foo(:bar)&quot; ) . inspected &quot;{:foo, [line: 1], [:bar]}&quot;","ref":"readme.html#features"},{"type":"extras","title":"Dune - Limitations","doc":"Dune supports a fair subset of the base language, but it cannot safely support advanced features (at least at this stage) such as: custom structs / behaviours / protocols concurrency / processes / OTP metaprogramming","ref":"readme.html#limitations"},{"type":"extras","title":"Dune - Security guarantees","doc":"Because of the approch being used, Dune cannot offer strong security guarantees, and should not be considered a sufficient security layer by itself. A best-effort approach is made to prevent attackers from getting outside of the original process and from calling any function/macro outside of the allowlist. However, it is impossible to prove that all escape paths have been completely blocked. Due to how the Erlang VM works, an attacker able to escape the sandbox could get full access to the VM without restriction. See the EEF guidelines about sandboxing for more information. Use at your own risk and avoid running it directly on a server with any sensitive access, e.g. to a database.","ref":"readme.html#security-guarantees"},{"type":"extras","title":"Dune - Installation","doc":"The package can be installed by adding dune to your list of dependencies in mix.exs : def deps do [ { :dune , &quot;~&gt; 0.3.2&quot; } ] end Documentation can be found at https://hexdocs.pm/dune .","ref":"readme.html#installation"},{"type":"extras","title":"Dune - FAQ","doc":"Why can I still create atoms? Atoms are converted by the parser and mapped to always use a given set of atoms. So when you type foo = bar(:baz) , the atoms :foo , :bar and :baz won't actually be created, but other atoms like :atom1 , :atom2 and :atom3 are going to be used instead. So even if many user run various codes using many different variable and function names, they will all pull from the same pool of atoms. Why can I still create modules? Modules are being defined globally within the VM, making it both an isolation concern and a memory concern. But modules are an important part of the Elixir language, and are especially important for learning platforms. Dune implements an alternative defmodule , relying on maps of anonymous functions at runtime and should reproduce the basic behavior of actual modules, with support for recursive and private functions. Why is the behavior different than Elixir when doing X? As explained above, some parts of the language are actually being completely reimplemented because the original version could not be safely sandboxed: atoms, modules... While these alternative implementation aim to be as close as possible to the original ones, they might differ in some cases, because the code being executed is actually different. Why can't I do X? Some parts of the language are being restricted because they present a direct security risk, while some other would need to be reimplemented in an alternative way and therefore need a consequent amount of work that hasn't been done yet.","ref":"readme.html#faq"},{"type":"extras","title":"Changelog","doc":"","ref":"changelog.html"},{"type":"extras","title":"Changelog - Dev","doc":"","ref":"changelog.html#dev"},{"type":"extras","title":"Changelog - v0.3.2 (2023-08-12)","doc":"Enhancements dbg/1 uses pretty printing Bug fixes Fix error message on restricted dbg/0","ref":"changelog.html#v0-3-2-2023-08-12"},{"type":"extras","title":"Changelog - v0.3.1 (2023-08-12)","doc":"Enhancements Add support for dbg/1 Bug fixes Properly distinguish user code throw/1 from internal ones","ref":"changelog.html#v0-3-1-2023-08-12"},{"type":"extras","title":"Changelog - v0.3.0 (2023-08-09)","doc":"Breaking changes Drop support for Elixir 1.13 Compile errors are now returned as a separate type :compile_error Enhancements Support Elixir 1.15 Capture compile diagnostics (Elixir &gt;= 1.15) Bug fixes Better handle UndefinedFunctionError for dynamic module names","ref":"changelog.html#v0-3-0-2023-08-09"},{"type":"extras","title":"Changelog - v0.2.6 (2022-10-17)","doc":"Enhancements Support Elixir 1.14","ref":"changelog.html#v0-2-6-2022-10-17"},{"type":"extras","title":"Changelog - v0.2.5 (2022-08-25)","doc":"Bug fixes Restrict the use of :counters in Dune.Allowlist.Default , since it can leak memory","ref":"changelog.html#v0-2-5-2022-08-25"},{"type":"extras","title":"Changelog - v0.2.4 (2022-07-13)","doc":"Bug fixes Validate module names in defmodule , reject nil or booleans","ref":"changelog.html#v0-2-4-2022-07-13"},{"type":"extras","title":"Changelog - v0.2.3 (2022-04-13)","doc":"Bug fixes Dune.string_to_quoted/2 quotes modules with . correctly OTP 25 regression: keep a clean stacktrace for exceptions","ref":"changelog.html#v0-2-3-2022-04-13"},{"type":"extras","title":"Changelog - v0.2.2 (2022-04-05)","doc":"Enhancements Add Dune.string_to_quoted/2 to make it possible to visualize AST Merged parsing and eval options in a single Dune.Opts for simplicity Add a pretty option to inspect result Better error message when def/2 and defp/2 called outside a module Breaking changes Removed Dune.Parser.Opts and Dune.Eval.Opts","ref":"changelog.html#v0-2-2-2022-04-05"},{"type":"extras","title":"Changelog - v0.2.1 (2022-03-19)","doc":"Bug fixes Handle default arguments in functions Handle conflicting def and defp with same name/arity","ref":"changelog.html#v0-2-1-2022-03-19"},{"type":"extras","title":"Changelog - v0.2.0 (2022-01-02)","doc":"Breaking changes Support Elixir 1.13, drop support for 1.12 This fixes a bug in atoms was due to the Elixir parser","ref":"changelog.html#v0-2-0-2022-01-02"},{"type":"extras","title":"Changelog - v0.1.2 (2021-10-17)","doc":"Enhancements Allow safe functions from the :erlang module Bug fixes Fix bug when calling custom function in nested AST","ref":"changelog.html#v0-1-2-2021-10-17"},{"type":"extras","title":"Changelog - v0.1.1 (2021-10-16)","doc":"Bug fixes Prevent atom leaks due to Code.string_to_quoted/2 not respecting static_atoms_encoder Handle Elixir 1.12 bug on single atom ASTs Handle atoms prefixed with Elixir. properly Fix inspect for quoted atoms","ref":"changelog.html#v0-1-1-2021-10-16"},{"type":"extras","title":"Changelog - v0.1.0 (2021-09-19)","doc":"Initial release","ref":"changelog.html#v0-1-0-2021-09-19"},{"type":"extras","title":"LICENSE","doc":"MIT License Copyright (c) 2021 Sabiwara Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}]